#include "train.h"

Train::Train(uint8_t id, uint8_t address, uint8_t functionMask, int8_t initalValue):
	Item(address, QString("Train ")+QString::number(id), initalValue),
	functionMask_(functionMask),
	train_id_(id)
{
}

Microcontroller *Train::micro = nullptr;

void Train::setFunction(uint8_t funciton, bool value)
{
	Item::setFunction(funciton, value);
	if(micro)
		micro->trainSetFunction(train_id_, funciton, value);
}

void Train::setValue(int8_t value)
{
	Item::setValue(value);
	if(suspended_ && value != 0)
	{
		suspended_ = false;
		unsuspended(id(), getDirection());
	}
	if(micro)
		micro->trainSetSpeed(train_id_, value);
}

void Train::informValue(int8_t value)
{
	if(suspended_ && value != 0)
	{
		suspended_ = false;
		unsuspended(id(), getDirection());
	}
	Item::informValue(value);
}

void Train::reverse()
{
	if(micro)
		micro->trainReverse(train_id_);
}

int Train::ownsTag(NfcUid uid)
{
	for(size_t i = 0; i < tags.size(); ++i)
	{
		if(uid == tags[i])
		{
			if(i == 0)
				return getDirection() == FORWARD ? TAG_FRONT : TAG_BACK;
			else if(i == tags.size()-1)
				return getDirection() == REVERSE ? TAG_FRONT : TAG_BACK;
			else
				return TAG_CENTER;
		}
	}
	return TAG_NONE;
}

bool Train::suspend()
{
	if(suspended_)
		return false;
	suspendedSpeed_ = value_;
	setValue(0);
	suspended_ = true;
	return true;
}

bool Train::resume()
{
	if(!suspended_)
		return false;
	suspended_ = false;
	setValue(suspendedSpeed_);
	return true;
}

bool Train::hasBackTag()
{
	return tags.size() > 1;
}

bool Train::suspendend()
{
	return suspended_;
}

int Train::getDirection()
{
	if(!suspended_)
		return value_ < 0 ? REVERSE : FORWARD;
	else
		return suspendedSpeed_ < 0 ? REVERSE : FORWARD;
}

